/*
 *  Copyright (C) 2008 Patrik Gornicz, Gornicz_P (at) hotmail (dot) com.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "PhysicsEntity.h"

#include <bear/debug.h>
#include <bear/Vector2.h>
using namespace bear;

#include "effectManager.h"


/// ***** Constructors/Destructors *****

PhysicsEntity::PhysicsEntity(const Vector2& pos)
    : Entity(pos), force(0,0), mass(1), CoR(0.8)
{

}
PhysicsEntity::~PhysicsEntity()
{

}

/// ***** Public Class Methods *****

void PhysicsEntity::update(float time_step)
{
    position = positionAt(time_step);
    velocity = velocityAt(time_step);

    force *= 0;
}

Vector2 PhysicsEntity::positionAt(float time_step) const
{
    Vector2 newPosition = position;
    Vector2 newVelocity = velocity;
    Vector2 newForce = force;

    newPosition += effect::positionDelta(this, time_step);
    newVelocity += effect::velocityDelta(this, time_step);
    newForce += effect::forceDelta(this, time_step);

    return newForce/mass / 2 * time_step * time_step + newVelocity * time_step + newPosition;
}

Vector2 PhysicsEntity::velocityAt(float time_step) const
{
    Vector2 newForce = force;
    Vector2 newVelocity = velocity;

    newForce += effect::forceDelta(this, time_step);
    newVelocity += effect::velocityDelta(this, time_step);

    return newForce/mass / 2 * time_step + newVelocity;
}

void PhysicsEntity::applyForce(const Vector2& force)
{
    applyForce(force, position);
}
void PhysicsEntity::applyForce(const Vector2& force, const Vector2& /*at*/)
{
    this->force += force;
}

void PhysicsEntity::applyImpulse(const Vector2& impulse)
{
    applyImpulse(impulse, position);
}
void PhysicsEntity::applyImpulse(const Vector2& impulse, const Vector2& /*at*/)
{
    velocity += impulse;
}

void PhysicsEntity::applyNudge(const Vector2& vecPush)
{
    position += vecPush;
}
